clc; 
clear all
close all

SNR_dB =[0:2:10];  
SNR_linear=10.^(SNR_dB/10.);
len = length(SNR_linear);
sample = 30; 
sparsity=8;
%%% system parameters
N = 512; % number of beams (transmit antennas)
L = 6; % number of all paths
gamma=1/2; 
Lf = L*gamma; % number of paths for far-field 
Ln = L*(1-gamma); % number of paths for near-field
M = 256; % number of pilot overhead

% fc = 30e9; % carrier frequency
fc = 30e9; % carrier frequency
c = 3e8;
lambda_c = c/fc; % wavelength 
d = lambda_c / 2; % antenna space

% the far-field angle-domain DFT matrix
Uf = (1/sqrt(N))*exp(-1i*pi*[0:N-1]'*[-(N-1)/2:1:(N/2)]*(2/N));

% the near-field polar-domain transform matrix [5]
Rmin=10;
Rmax=100;
eta = 2.5; 
[Un, label, dict_cell, label_cell] = QuaCode(N, d, lambda_c, eta, Rmin, Rmax);
error_hsdomp=zeros(sample,len);
energy=zeros(sample,1);


for s=1:sample
    s
    [h,hf,hn,theta_f,theta_n,r_n] = generate_hybrid_field_channel(N, Lf, Ln, d, fc,Rmin, Rmax);
    
    for iS=1:len
        sigma2=1/SNR_linear(iS);
        P=((rand(M,N)>0.5)*2-1)/sqrt(M); % pilot matrix
        noise = sqrt(sigma2)*(randn(M,1)+1i*randn(M,1))/sqrt(2);
        y=P*h+noise;
        
        %% the proposed hybrid-field SD-OMP based scheme
        hhat_hsdomp=SD_OMP(y,P,Uf,Un,Lf,sparsity*Ln);
        error_hsdomp(s,iS)=sum(abs(hhat_hsdomp-h).^2);
           
    end
    energy(s)=sum(abs(h).^2);  
end
 
nmse_hsdomp = mean(error_hsdomp)/mean(energy) 
nmse_hsdomp = 10*log10(nmse_hsdomp)

figure('color',[1,1,1]); 
ha=gca;
plot(SNR_dB,nmse_hsdomp,'rs-','linewidth',1.5);
hold on
grid on
legend('Hybrid-field SD-OMP')
xlabel('SNR (dB)')
ylabel('NMSE (dB)')
hold off